@@ -782,6 +782,8 @@ TEST_CASE_METHOD(
     GroupFx,
     "C API: Group, write/read with named members",
     "[capi][group][metadata][read]") {
+  bool remove_by_name = GENERATE(true, false);
+
   // Create and open group in write mode
   // TODO: refactor for each supported FS.
   std::string temp_dir = fs_vec_[0]->temp_dir();
@@ -893,13 +895,25 @@ TEST_CASE_METHOD(
   rc = tiledb_group_open(ctx_, group2, TILEDB_WRITE);
   REQUIRE(rc == TILEDB_OK);
 
-  rc = tiledb_group_remove_member(ctx_, group1, group2_uri.c_str());
+  // Remove by bame or URI.
+  if (remove_by_name) {
+    rc = tiledb_group_remove_member(ctx_, group1, "four");
+  } else {
+    rc = tiledb_group_remove_member(ctx_, group1, group2_uri.c_str());
+  }
   REQUIRE(rc == TILEDB_OK);
+
   // Group is the latest element
   group1_expected.resize(group1_expected.size() - 1);
 
-  rc = tiledb_group_remove_member(ctx_, group2, array3_uri.c_str());
+  // Remove by bame or URI.
+  if (remove_by_name) {
+    rc = tiledb_group_remove_member(ctx_, group2, "three");
+  } else {
+    rc = tiledb_group_remove_member(ctx_, group2, array3_uri.c_str());
+  }
   REQUIRE(rc == TILEDB_OK);
+
   // There should be nothing left in group2
   group2_expected.clear();
 
@@ -936,6 +950,78 @@ TEST_CASE_METHOD(
   remove_temp_dir(temp_dir);
 }
 
+TEST_CASE_METHOD(
+    GroupFx,
+    "C API: Group, write duplicate members",
+    "[capi][group][write][duplicate]") {
+  // Create and open group in write mode
+  // TODO: refactor for each supported FS.
+  std::string temp_dir = fs_vec_[0]->temp_dir();
+  create_temp_dir(temp_dir);
+
+  const tiledb::sm::URI group1_uri(temp_dir + "group1");
+  const tiledb::sm::URI group2_uri(temp_dir + "group1/group2");
+  const tiledb::sm::URI group3_uri(temp_dir + "group1/group3");
+
+  REQUIRE(tiledb_group_create(ctx_, group1_uri.c_str()) == TILEDB_OK);
+  REQUIRE(tiledb_group_create(ctx_, group2_uri.c_str()) == TILEDB_OK);
+  REQUIRE(tiledb_group_create(ctx_, group3_uri.c_str()) == TILEDB_OK);
+
+  tiledb_group_t* group1;
+  int rc = tiledb_group_alloc(ctx_, group1_uri.c_str(), &group1);
+  REQUIRE(rc == TILEDB_OK);
+  set_group_timestamp(group1, 1);
+  rc = tiledb_group_open(ctx_, group1, TILEDB_WRITE);
+  REQUIRE(rc == TILEDB_OK);
+
+  SECTION("Name-name collision") {
+    rc =
+        tiledb_group_add_member(ctx_, group1, group2_uri.c_str(), false, "one");
+    REQUIRE(rc == TILEDB_OK);
+    rc =
+        tiledb_group_add_member(ctx_, group1, group3_uri.c_str(), false, "one");
+    REQUIRE(rc == TILEDB_ERR);
+    rc = tiledb_group_remove_member(ctx_, group1, "one");
+    REQUIRE(rc == TILEDB_ERR);
+  }
+
+  SECTION("Name-URI collision") {
+    rc = tiledb_group_add_member(
+        ctx_, group1, group2_uri.c_str(), false, "group2");
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(ctx_, group1, "group2", true, nullptr);
+    REQUIRE(rc == TILEDB_ERR);
+    rc = tiledb_group_remove_member(ctx_, group1, "group2");
+    REQUIRE(rc == TILEDB_ERR);
+  }
+
+  SECTION("URI-name collision") {
+    rc = tiledb_group_add_member(ctx_, group1, "group2", true, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group1, group3_uri.c_str(), false, "group2");
+    REQUIRE(rc == TILEDB_ERR);
+    rc = tiledb_group_remove_member(ctx_, group1, "group2");
+    REQUIRE(rc == TILEDB_ERR);
+  }
+
+  SECTION("URI-URI collision") {
+    rc = tiledb_group_add_member(
+        ctx_, group1, group2_uri.c_str(), false, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group1, group2_uri.c_str(), false, nullptr);
+    REQUIRE(rc == TILEDB_ERR);
+    rc = tiledb_group_remove_member(ctx_, group1, group2_uri.c_str());
+    REQUIRE(rc == TILEDB_ERR);
+  }
+
+  // Close group
+  rc = tiledb_group_close(ctx_, group1);
+  tiledb_group_free(&group1);
+  remove_temp_dir(temp_dir);
+}
+
 TEST_CASE_METHOD(
     GroupFx,
     "C API: Group, consolidation",
@@ -1197,6 +1283,8 @@ TEST_CASE_METHOD(
     GroupFx,
     "C API: Group, write/read, relative with named members",
     "[capi][group][metadata][read]") {
+  bool remove_by_name = GENERATE(true, false);
+
   // Create and open group in write mode
   // TODO: refactor for each supported FS.
   std::string temp_dir = fs_vec_[0]->temp_dir();
@@ -1321,12 +1409,22 @@ TEST_CASE_METHOD(
   rc = tiledb_group_open(ctx_, group2, TILEDB_WRITE);
   REQUIRE(rc == TILEDB_OK);
 
-  rc = tiledb_group_remove_member(ctx_, group1, group2_uri.c_str());
+  // Remove by bame or URI.
+  if (remove_by_name) {
+    rc = tiledb_group_remove_member(ctx_, group1, "eight");
+  } else {
+    rc = tiledb_group_remove_member(ctx_, group1, group2_uri.c_str());
+  }
   REQUIRE(rc == TILEDB_OK);
   // Group is the latest element
   group1_expected.resize(group1_expected.size() - 1);
 
-  rc = tiledb_group_remove_member(ctx_, group2, array3_relative_uri.c_str());
+  // Remove by bame or URI.
+  if (remove_by_name) {
+    rc = tiledb_group_remove_member(ctx_, group2, "seven");
+  } else {
+    rc = tiledb_group_remove_member(ctx_, group2, array3_relative_uri.c_str());
+  }
   REQUIRE(rc == TILEDB_OK);
   // There should be nothing left in group2
   group2_expected.clear();
@@ -1678,4 +1776,72 @@ TEST_CASE_METHOD(
   tiledb_group_free(&group4_read);
   remove_temp_dir(temp_dir);
 }
+
+TEST_CASE_METHOD(
+    GroupFx,
+    "C API: Group metadata serialization",
+    "[capi][group][metadata][serialization]") {
+  std::string temp_dir = fs_vec_[0]->temp_dir();
+  create_temp_dir(temp_dir);
+
+  tiledb::sm::URI group_uri(temp_dir + "group");
+  REQUIRE(tiledb_group_create(ctx_, group_uri.c_str()) == TILEDB_OK);
+  tiledb::sm::URI group_deserialized_uri(temp_dir + "group_deserialized");
+  REQUIRE(
+      tiledb_group_create(ctx_, group_deserialized_uri.c_str()) == TILEDB_OK);
+
+  tiledb_group_t* group = nullptr;
+  REQUIRE(tiledb_group_alloc(ctx_, group_uri.c_str(), &group) == TILEDB_OK);
+  REQUIRE(tiledb_group_open(ctx_, group, TILEDB_WRITE) == TILEDB_OK);
+
+  int value = 123;
+  REQUIRE(
+      tiledb_group_put_metadata(
+          ctx_, group, "testmetadata", TILEDB_INT32, 1, &value) == TILEDB_OK);
+
+  REQUIRE(tiledb_group_close(ctx_, group) == TILEDB_OK);
+  tiledb_group_free(&group);
+
+  // Reopen in read mode
+  REQUIRE(tiledb_group_alloc(ctx_, group_uri.c_str(), &group) == TILEDB_OK);
+  REQUIRE(tiledb_group_open(ctx_, group, TILEDB_READ) == TILEDB_OK);
+
+  tiledb_group_t* group_deserialized = nullptr;
+  REQUIRE(
+      tiledb_group_alloc(
+          ctx_, group_deserialized_uri.c_str(), &group_deserialized) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_group_open(ctx_, group_deserialized, TILEDB_WRITE) == TILEDB_OK);
+
+  REQUIRE(
+      tiledb_group_serialize(ctx_, group, group_deserialized, TILEDB_CAPNP) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_group_close(ctx_, group_deserialized) == TILEDB_OK);
+  tiledb_group_free(&group_deserialized);
+
+  // Reopen in read mode
+  REQUIRE(
+      tiledb_group_alloc(
+          ctx_, group_deserialized_uri.c_str(), &group_deserialized) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_group_open(ctx_, group_deserialized, TILEDB_READ) == TILEDB_OK);
+
+  tiledb_datatype_t dtype;
+  uint32_t num;
+  const void* val;
+  tiledb_group_get_metadata(
+      ctx_, group_deserialized, "testmetadata", &dtype, &num, &val);
+
+  REQUIRE(val != nullptr);
+  CHECK(*static_cast<const int32_t*>(val) == 123);
+
+  REQUIRE(tiledb_group_close(ctx_, group) == TILEDB_OK);
+  tiledb_group_free(&group);
+  REQUIRE(tiledb_group_close(ctx_, group_deserialized) == TILEDB_OK);
+  tiledb_group_free(&group_deserialized);
+  remove_temp_dir(temp_dir);
+}
 #endif
