@@ -552,29 +552,36 @@ void Array::delete_fragments(
   ensure_array_is_valid_for_delete(uri);
 
   // Delete fragments
-  // #TODO Add rest support for delete_fragments
   if (remote_) {
-    throw ArrayException(
-        "[delete_fragments] Remote arrays currently unsupported.");
+    auto rest_client = resources_.rest_client();
+    if (rest_client == nullptr) {
+      throw ArrayException(
+          "[delete_fragments] Remote array with no REST client.");
+    }
+    rest_client->post_delete_fragments_to_rest(
+        uri, this, timestamp_start, timestamp_end);
   } else {
     storage_manager_->delete_fragments(
         uri.c_str(), timestamp_start, timestamp_end);
   }
 }
 
-void Array::delete_fragments_list(
-    const URI& uri, const std::vector<URI>& fragment_uris) {
+void Array::delete_fragments_list(const std::vector<URI>& fragment_uris) {
   // Check that data deletion is allowed
-  ensure_array_is_valid_for_delete(uri);
+  ensure_array_is_valid_for_delete(array_uri_);
 
-  // Delete fragments
-  // #TODO Add rest support for delete_fragments_list
+  // Delete fragments_list
   if (remote_) {
-    throw ArrayException(
-        "[delete_fragments_list] Remote arrays currently unsupported.");
+    auto rest_client = resources_.rest_client();
+    if (rest_client == nullptr) {
+      throw ArrayException(
+          "[delete_fragments_list] Remote array with no REST client.");
+    }
+    rest_client->post_delete_fragments_list_to_rest(
+        array_uri_, this, fragment_uris);
   } else {
     auto array_dir = ArrayDirectory(
-        resources_, uri, 0, std::numeric_limits<uint64_t>::max());
+        resources_, array_uri_, 0, std::numeric_limits<uint64_t>::max());
     array_dir.delete_fragments_list(fragment_uris);
   }
 }
@@ -608,8 +615,7 @@ std::vector<shared_ptr<const Enumeration>> Array::get_enumerations(
       auto rest_client = resources_.rest_client();
       if (rest_client == nullptr) {
         throw ArrayException(
-            "Error loading enumerations; "
-            "Remote array with no REST client.");
+            "Error loading enumerations; Remote array with no REST client.");
       }
 
       std::vector<std::string> names_to_load;
@@ -633,7 +639,7 @@ std::vector<shared_ptr<const Enumeration>> Array::get_enumerations(
 
       // Load the enumerations from storage
       loaded = array_dir_.load_enumerations_from_paths(
-          paths_to_load, get_encryption_key());
+          paths_to_load, get_encryption_key(), memory_tracker_);
     }
 
     // Store the loaded enumerations in the schema
@@ -906,103 +912,89 @@ void Array::set_config(Config config) {
   config_.inherit(config);
 }
 
-Status Array::delete_metadata(const char* key) {
+void Array::delete_metadata(const char* key) {
   // Check if array is open
   if (!is_open_) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot delete metadata. Array is not open"));
+    throw ArrayException("Cannot delete metadata. Array is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::WRITE &&
       query_type_ != QueryType::MODIFY_EXCLUSIVE) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot delete metadata. Array was "
-                          "not opened in write or modify_exclusive mode"));
+    throw ArrayException(
+        "Cannot delete metadata. Array was not opened in write or "
+        "modify_exclusive mode");
   }
 
   // Check if key is null
   if (key == nullptr) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot delete metadata. Key cannot be null"));
+    throw ArrayException("Cannot delete metadata. Key cannot be null");
   }
 
-  RETURN_NOT_OK(metadata_.del(key));
-
-  return Status::Ok();
+  metadata_.del(key);
 }
 
-Status Array::put_metadata(
+void Array::put_metadata(
     const char* key,
     Datatype value_type,
     uint32_t value_num,
     const void* value) {
   // Check if array is open
   if (!is_open_) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot put metadata; Array is not open"));
+    throw ArrayException("Cannot put metadata; Array is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::WRITE &&
       query_type_ != QueryType::MODIFY_EXCLUSIVE) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot put metadata; Array was "
-                          "not opened in write or modify_exclusive mode"));
+    throw ArrayException(
+        "Cannot put metadata; Array was not opened in write or "
+        "modify_exclusive mode");
   }
 
   // Check if key is null
   if (key == nullptr) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot put metadata; Key cannot be null"));
+    throw ArrayException("Cannot put metadata; Key cannot be null");
   }
 
   // Check if value type is ANY
   if (value_type == Datatype::ANY) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot put metadata; Value type cannot be ANY"));
+    throw ArrayException("Cannot put metadata; Value type cannot be ANY");
   }
 
-  RETURN_NOT_OK(metadata_.put(key, value_type, value_num, value));
-
-  return Status::Ok();
+  metadata_.put(key, value_type, value_num, value);
 }
 
-Status Array::get_metadata(
+void Array::get_metadata(
     const char* key,
     Datatype* value_type,
     uint32_t* value_num,
     const void** value) {
   // Check if array is open
   if (!is_open_) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get metadata; Array is not open"));
+    throw ArrayException("Cannot get metadata; Array is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::READ) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get metadata; Array was "
-                          "not opened in read mode"));
+    throw ArrayException(
+        "Cannot get metadata; Array was not opened in read mode");
   }
 
   // Check if key is null
   if (key == nullptr) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get metadata; Key cannot be null"));
+    throw ArrayException("Cannot get metadata; Key cannot be null");
   }
 
   // Load array metadata, if not loaded yet
   if (!metadata_loaded_) {
-    RETURN_NOT_OK(load_metadata());
+    throw_if_not_ok(load_metadata());
   }
 
-  RETURN_NOT_OK(metadata_.get(key, value_type, value_num, value));
-
-  return Status::Ok();
+  metadata_.get(key, value_type, value_num, value);
 }
 
-Status Array::get_metadata(
+void Array::get_metadata(
     uint64_t index,
     const char** key,
     uint32_t* key_len,
@@ -1011,81 +1003,66 @@ Status Array::get_metadata(
     const void** value) {
   // Check if array is open
   if (!is_open_) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get metadata; Array is not open"));
+    throw ArrayException("Cannot get metadata; Array is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::READ) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get metadata; Array was "
-                          "not opened in read mode"));
+    throw ArrayException(
+        "Cannot get metadata; Array was not opened in read mode");
   }
 
   // Load array metadata, if not loaded yet
   if (!metadata_loaded_) {
-    RETURN_NOT_OK(load_metadata());
+    throw_if_not_ok(load_metadata());
   }
 
-  RETURN_NOT_OK(
-      metadata_.get(index, key, key_len, value_type, value_num, value));
-
-  return Status::Ok();
+  metadata_.get(index, key, key_len, value_type, value_num, value);
 }
 
-Status Array::get_metadata_num(uint64_t* num) {
+uint64_t Array::metadata_num() {
   // Check if array is open
   if (!is_open_) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get number of metadata; Array is not open"));
+    throw ArrayException("Cannot get number of metadata; Array is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::READ) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get number of metadata; Array was "
-                          "not opened in read mode"));
+    throw ArrayException(
+        "Cannot get number of metadata; Array was not opened in read mode");
   }
 
   // Load array metadata, if not loaded yet
   if (!metadata_loaded_) {
-    RETURN_NOT_OK(load_metadata());
+    throw_if_not_ok(load_metadata());
   }
 
-  *num = metadata_.num();
-
-  return Status::Ok();
+  return metadata_.num();
 }
 
-Status Array::has_metadata_key(
-    const char* key, Datatype* value_type, bool* has_key) {
+std::optional<Datatype> Array::metadata_type(const char* key) {
   // Check if array is open
   if (!is_open_) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get metadata; Array is not open"));
+    throw ArrayException("Cannot get metadata; Array is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::READ) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get metadata; Array was "
-                          "not opened in read mode"));
+    throw ArrayException(
+        "Cannot get metadata; Array was not opened in read mode");
   }
 
   // Check if key is null
   if (key == nullptr) {
-    return LOG_STATUS(
-        Status_ArrayError("Cannot get metadata; Key cannot be null"));
+    throw ArrayException("Cannot get metadata; Key cannot be null");
   }
 
   // Load array metadata, if not loaded yet
   if (!metadata_loaded_) {
-    RETURN_NOT_OK(load_metadata());
+    throw_if_not_ok(load_metadata());
   }
 
-  RETURN_NOT_OK(metadata_.has_key(key, value_type, has_key));
-
-  return Status::Ok();
+  return metadata_.metadata_type(key);
 }
 
 Metadata* Array::unsafe_metadata() {
