@@ -233,13 +233,11 @@ class Array {
   /**
    * Deletes the fragments with the given URIs from the Array with given URI.
    *
-   * @param uri The uri of the Array whose fragments are to be deleted.
    * @param fragment_uris The uris of the fragments to be deleted.
    *
    * @pre The Array must be open for exclusive writes
    */
-  void delete_fragments_list(
-      const URI& uri, const std::vector<URI>& fragment_uris);
+  void delete_fragments_list(const std::vector<URI>& fragment_uris);
 
   /** Returns a constant pointer to the encryption key. */
   const EncryptionKey* encryption_key() const;
@@ -414,9 +412,8 @@ class Array {
    * Deletes metadata from an array opened in WRITE mode.
    *
    * @param key The key of the metadata item to be deleted.
-   * @return Status
    */
-  Status delete_metadata(const char* key);
+  void delete_metadata(const char* key);
 
   /**
    * Puts metadata into an array opened in WRITE mode.
@@ -428,9 +425,8 @@ class Array {
    *     same datatype. This argument indicates the number of items in the
    *     value component of the metadata.
    * @param value The metadata value in binary form.
-   * @return Status
    */
-  Status put_metadata(
+  void put_metadata(
       const char* key,
       Datatype value_type,
       uint32_t value_num,
@@ -447,9 +443,8 @@ class Array {
    *     same datatype. This argument indicates the number of items in the
    *     value component of the metadata.
    * @param value The metadata value in binary form.
-   * @return Status
    */
-  Status get_metadata(
+  void get_metadata(
       const char* key,
       Datatype* value_type,
       uint32_t* value_num,
@@ -466,9 +461,8 @@ class Array {
    *     same datatype. This argument indicates the number of items in the
    *     value component of the metadata.
    * @param value The metadata value in binary form.
-   * @return Status
    */
-  Status get_metadata(
+  void get_metadata(
       uint64_t index,
       const char** key,
       uint32_t* key_len,
@@ -477,10 +471,10 @@ class Array {
       const void** value);
 
   /** Returns the number of array metadata items. */
-  Status get_metadata_num(uint64_t* num);
+  uint64_t metadata_num();
 
-  /** Sets has_key == 1 and corresponding value_type if the array has key. */
-  Status has_metadata_key(const char* key, Datatype* value_type, bool* has_key);
+  /** Gets the type of the given metadata or nullopt if it does not exist. */
+  std::optional<Datatype> metadata_type(const char* key);
 
   /** Retrieves the array metadata object. */
   Status metadata(Metadata** metadata);
