@@ -33,6 +33,7 @@
 #ifndef TILEDB_ARRAY_DIRECTORY_H
 #define TILEDB_ARRAY_DIRECTORY_H
 
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/common/status.h"
 #include "tiledb/common/thread_pool.h"
 #include "tiledb/sm/array_schema/array_schema.h"
@@ -394,7 +395,8 @@ class ArrayDirectory {
    */
   std::vector<shared_ptr<const Enumeration>> load_enumerations_from_paths(
       const std::vector<std::string>& enumeration_paths,
-      const EncryptionKey& encryption_key) const;
+      const EncryptionKey& encryption_key,
+      MemoryTracker& memory_tracker) const;
 
   /** Returns the array URI. */
   const URI& uri() const;
@@ -826,7 +828,8 @@ class ArrayDirectory {
    */
   shared_ptr<const Enumeration> load_enumeration(
       const std::string& enumeration_path,
-      const EncryptionKey& encryption_key) const;
+      const EncryptionKey& encryption_key,
+      MemoryTracker& memory_tracker) const;
 };
 
 }  // namespace sm
