@@ -44,6 +44,8 @@
 #include "tiledb/api/c_api/attribute/attribute_api_external_experimental.h"
 #include "tiledb/api/c_api/enumeration/enumeration_api_experimental.h"
 #include "tiledb/api/c_api/group/group_api_external_experimental.h"
+#include "tiledb/api/c_api/query_aggregate/query_aggregate_api_external_experimental.h"
+#include "tiledb/api/c_api/query_field/query_field_api_external_experimental.h"
 #include "tiledb/api/c_api/query_plan/query_plan_api_external_experimental.h"
 #include "tiledb_dimension_label_experimental.h"
 
@@ -73,7 +75,7 @@ extern "C" {
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT capi_return_t
-tiledb_log_warn(tiledb_ctx_t* ctx, const char* message);
+tiledb_log_warn(tiledb_ctx_t* ctx, const char* message) TILEDB_NOEXCEPT;
 
 /* ********************************* */
 /*              AS BUILT             */
