@@ -84,6 +84,19 @@ enum class VFSMode : uint8_t;
 
 /** The VFS configuration parameters. */
 struct VFSParameters {
+  /* ********************************* */
+  /*          TYPE DEFINITIONS         */
+  /* ********************************* */
+
+  enum struct ReadLoggingMode {
+    DISABLED,
+    FRAGMENTS,
+    FRAGMENT_FILES,
+    ALL_FILES,
+    ALL_READS,
+    ALL_READS_ALWAYS
+  };
+
   VFSParameters() = delete;
 
   VFSParameters(const Config& config)
@@ -91,7 +104,30 @@ struct VFSParameters {
             config.get<uint64_t>("vfs.min_parallel_size").value())
       , read_ahead_cache_size_(
             config.get<uint64_t>("vfs.read_ahead_cache_size").value())
-      , read_ahead_size_(config.get<uint64_t>("vfs.read_ahead_size").value()){};
+      , read_ahead_size_(config.get<uint64_t>("vfs.read_ahead_size").value())
+      , read_logging_mode_(ReadLoggingMode::DISABLED) {
+    auto log_mode = config.get<std::string>("vfs.read_logging_mode").value();
+    if (log_mode == "") {
+      read_logging_mode_ = ReadLoggingMode::DISABLED;
+    } else if (log_mode == "fragments") {
+      read_logging_mode_ = ReadLoggingMode::FRAGMENTS;
+    } else if (log_mode == "fragment_files") {
+      read_logging_mode_ = ReadLoggingMode::FRAGMENT_FILES;
+    } else if (log_mode == "all_files") {
+      read_logging_mode_ = ReadLoggingMode::ALL_FILES;
+    } else if (log_mode == "all_reads") {
+      read_logging_mode_ = ReadLoggingMode::ALL_READS;
+    } else if (log_mode == "all_reads_always") {
+      read_logging_mode_ = ReadLoggingMode::ALL_READS_ALWAYS;
+    } else {
+      std::stringstream ss;
+      ss << "Invalid read logging mode '" << log_mode << "'. "
+         << "Use one of 'fragments', 'fragment_files', 'all_files', "
+         << "'all_reads', or 'all_reads_always'. Read logging is currently "
+         << "disabled.";
+      LOG_WARN(ss.str());
+    }
+  };
 
   ~VFSParameters() = default;
 
@@ -103,13 +139,47 @@ struct VFSParameters {
 
   /** The byte size to read-ahead for each read. */
   uint64_t read_ahead_size_;
+
+  /** The read logging mode to use. */
+  ReadLoggingMode read_logging_mode_;
 };
 
+/** The base parameters for class VFS. */
+struct VFSBase {
+  VFSBase() = delete;
+
+  VFSBase(stats::Stats* const parent_stats)
+      : stats_(parent_stats->create_child("VFS")){};
+
+  ~VFSBase() = default;
+
+ protected:
+  /** The class stats. */
+  stats::Stats* stats_;
+};
+
+/** The S3 filesystem. */
+#ifdef HAVE_S3
+struct S3_within_VFS {
+  S3 s3_;
+  template <typename... Args>
+  S3_within_VFS(Args&&... args)
+      : s3_(std::forward<Args>(args)...) {
+  }
+};
+#else
+struct S3_within_VFS {
+  template <typename... Args>
+  S3_within_VFS(Args&&...) {
+  }  // empty constructor
+};
+#endif
+
 /**
  * This class implements a virtual filesystem that directs filesystem-related
  * function execution to the appropriate backend based on the input URI.
  */
-class VFS {
+class VFS : private VFSBase, S3_within_VFS {
  public:
   /* ********************************* */
   /*          TYPE DEFINITIONS         */
@@ -682,10 +752,6 @@ class VFS {
   GCS gcs_;
 #endif
 
-#ifdef HAVE_S3
-  S3 s3_;
-#endif
-
 #ifdef _WIN32
   Win win_;
 #else
@@ -696,9 +762,6 @@ class VFS {
   tdb_unique_ptr<hdfs::HDFS> hdfs_;
 #endif
 
-  /** The class stats. */
-  stats::Stats* stats_;
-
   /** The in-memory filesystem which is always supported */
   MemFilesystem memfs_;
 
@@ -726,9 +789,12 @@ class VFS {
   /** The read-ahead cache. */
   tdb_unique_ptr<ReadAheadCache> read_ahead_cache_;
 
-  /* The VFS configuration parameters. */
+  /** The VFS configuration parameters. */
   VFSParameters vfs_params_;
 
+  /** The URIs previously read by this instance. */
+  std::unordered_set<std::string> reads_logged_;
+
   /* ********************************* */
   /*          PRIVATE METHODS          */
   /* ********************************* */
@@ -775,6 +841,16 @@ class VFS {
    * read.
    */
   Status max_parallel_ops(const URI& uri, uint64_t* ops) const;
+
+  /**
+   * Log a read operation. The format of the log message depends on the
+   * config value `vfs.read_logging_mode`.
+   *
+   * @param uri The URI being read.
+   * @param offset The offset being read from.
+   * @param nbytes The number of bytes requested.
+   */
+  void log_read(const URI& uri, uint64_t offset, uint64_t nbytes);
 };
 
 }  // namespace tiledb::sm
