@@ -259,10 +259,10 @@ Status Group::close() {
       try {
         // If changes haven't been applied, apply them
         if (!changes_applied_) {
-          RETURN_NOT_OK(group_details_->apply_pending_changes());
+          group_details_->apply_pending_changes();
           changes_applied_ = group_details_->changes_applied();
         }
-        RETURN_NOT_OK(storage_manager_->group_close_for_writes(this));
+        throw_if_not_ok(storage_manager_->group_close_for_writes(this));
       } catch (StatusException& exc) {
         std::string msg = exc.what();
         msg += " : Was storage for the group moved or deleted before closing?";
@@ -346,91 +346,84 @@ void Group::delete_group(const URI& uri, bool recursive) {
   }
   // Clear metadata and other pending changes to avoid patching a deleted group.
   metadata_.clear();
-  throw_if_not_ok(group_details_->clear());
+  group_details_->clear();
 
   // Close the deleted group
   throw_if_not_ok(this->close());
 }
 
-Status Group::delete_metadata(const char* key) {
+void Group::delete_metadata(const char* key) {
   // Check if group is open
   if (!is_open_)
-    return Status_GroupError("Cannot delete metadata. Group is not open");
+    throw GroupStatusException("Cannot delete metadata. Group is not open");
 
   // Check mode
   if (query_type_ != QueryType::WRITE &&
       query_type_ != QueryType::MODIFY_EXCLUSIVE)
-    return Status_GroupError(
-        "Cannot delete metadata. Group was "
-        "not opened in write or modify_exclusive mode");
+    throw GroupStatusException(
+        "Cannot delete metadata. Group was not opened in write or "
+        "modify_exclusive mode");
 
   // Check if key is null
   if (key == nullptr)
-    return Status_GroupError("Cannot delete metadata. Key cannot be null");
+    throw GroupStatusException("Cannot delete metadata. Key cannot be null");
 
-  RETURN_NOT_OK(metadata_.del(key));
-
-  return Status::Ok();
+  metadata_.del(key);
 }
 
-Status Group::put_metadata(
+void Group::put_metadata(
     const char* key,
     Datatype value_type,
     uint32_t value_num,
     const void* value) {
   // Check if group is open
   if (!is_open_)
-    return Status_GroupError("Cannot put metadata; Group is not open");
+    throw GroupStatusException("Cannot put metadata; Group is not open");
 
   // Check mode
   if (query_type_ != QueryType::WRITE &&
       query_type_ != QueryType::MODIFY_EXCLUSIVE)
-    return Status_GroupError(
-        "Cannot put metadata; Group was "
-        "not opened in write or modify_exclusive mode");
+    throw GroupStatusException(
+        "Cannot put metadata; Group was not opened in write or "
+        "modify_exclusive mode");
 
   // Check if key is null
   if (key == nullptr)
-    return Status_GroupError("Cannot put metadata; Key cannot be null");
+    throw GroupStatusException("Cannot put metadata; Key cannot be null");
 
   // Check if value type is ANY
   if (value_type == Datatype::ANY)
-    return Status_GroupError("Cannot put metadata; Value type cannot be ANY");
+    throw GroupStatusException("Cannot put metadata; Value type cannot be ANY");
 
-  RETURN_NOT_OK(metadata_.put(key, value_type, value_num, value));
-
-  return Status::Ok();
+  metadata_.put(key, value_type, value_num, value);
 }
 
-Status Group::get_metadata(
+void Group::get_metadata(
     const char* key,
     Datatype* value_type,
     uint32_t* value_num,
     const void** value) {
   // Check if group is open
   if (!is_open_)
-    return Status_GroupError("Cannot get metadata; Group is not open");
+    throw GroupStatusException("Cannot get metadata; Group is not open");
 
   // Check mode
   if (query_type_ != QueryType::READ)
-    return Status_GroupError(
-        "Cannot get metadata; Group was "
-        "not opened in read mode");
+    throw GroupStatusException(
+        "Cannot get metadata; Group was not opened in read mode");
 
   // Check if key is null
   if (key == nullptr)
-    return Status_GroupError("Cannot get metadata; Key cannot be null");
+    throw GroupStatusException("Cannot get metadata; Key cannot be null");
 
   // Load group metadata, if not loaded yet
   if (!metadata_loaded_)
-    RETURN_NOT_OK(load_metadata());
+    load_metadata();
 
-  RETURN_NOT_OK(metadata_.get(key, value_type, value_num, value));
-
-  return Status::Ok();
+  metadata_.get(key, value_type, value_num, value);
 }
 
-Status Group::get_metadata(
+void Group::get_metadata(
     uint64_t index,
     const char** key,
     uint32_t* key_len,
@@ -439,68 +432,57 @@ Status Group::get_metadata(
     const void** value) {
   // Check if group is open
   if (!is_open_)
-    return Status_GroupError("Cannot get metadata; Group is not open");
+    throw GroupStatusException("Cannot get metadata; Group is not open");
 
   // Check mode
   if (query_type_ != QueryType::READ)
-    return Status_GroupError(
-        "Cannot get metadata; Group was "
-        "not opened in read mode");
+    throw GroupStatusException(
+        "Cannot get metadata; Group was not opened in read mode");
 
   // Load group metadata, if not loaded yet
   if (!metadata_loaded_)
-    RETURN_NOT_OK(load_metadata());
+    load_metadata();
 
-  RETURN_NOT_OK(
-      metadata_.get(index, key, key_len, value_type, value_num, value));
-
-  return Status::Ok();
+  metadata_.get(index, key, key_len, value_type, value_num, value);
 }
 
-Status Group::get_metadata_num(uint64_t* num) {
+uint64_t Group::get_metadata_num() {
   // Check if group is open
   if (!is_open_)
-    return Status_GroupError(
+    throw GroupStatusException(
         "Cannot get number of metadata; Group is not open");
 
   // Check mode
   if (query_type_ != QueryType::READ)
-    return Status_GroupError(
-        "Cannot get number of metadata; Group was "
-        "not opened in read mode");
+    throw GroupStatusException(
+        "Cannot get number of metadata; Group was not opened in read mode");
 
   // Load group metadata, if not loaded yet
   if (!metadata_loaded_)
-    RETURN_NOT_OK(load_metadata());
+    load_metadata();
 
-  *num = metadata_.num();
-
-  return Status::Ok();
+  return metadata_.num();
 }
 
-Status Group::has_metadata_key(
-    const char* key, Datatype* value_type, bool* has_key) {
+std::optional<Datatype> Group::metadata_type(const char* key) {
   // Check if group is open
   if (!is_open_)
-    return Status_GroupError("Cannot get metadata; Group is not open");
+    throw GroupStatusException("Cannot get metadata; Group is not open");
 
   // Check mode
   if (query_type_ != QueryType::READ)
-    return Status_GroupError(
-        "Cannot get metadata; Group was "
-        "not opened in read mode");
+    throw GroupStatusException(
+        "Cannot get metadata; Group was not opened in read mode");
 
   // Check if key is null
   if (key == nullptr)
-    return Status_GroupError("Cannot get metadata; Key cannot be null");
+    throw GroupStatusException("Cannot get metadata; Key cannot be null");
 
   // Load group metadata, if not loaded yet
   if (!metadata_loaded_)
-    RETURN_NOT_OK(load_metadata());
+    load_metadata();
 
-  RETURN_NOT_OK(metadata_.has_key(key, value_type, has_key));
-
-  return Status::Ok();
+  return metadata_.metadata_type(key);
 }
 
 Metadata* Group::unsafe_metadata() {
@@ -514,7 +496,7 @@ const Metadata* Group::metadata() const {
 Status Group::metadata(Metadata** metadata) {
   // Load group metadata, if not loaded yet
   if (!metadata_loaded_)
-    RETURN_NOT_OK(load_metadata());
+    load_metadata();
 
   *metadata = &metadata_;
 
@@ -545,7 +527,8 @@ void Group::set_config(Config config) {
 }
 
 Status Group::clear() {
-  return group_details_->clear();
+  group_details_->clear();
+  return Status::Ok();
 }
 
 void Group::add_member(const shared_ptr<GroupMember> group_member) {
@@ -575,15 +558,12 @@ Status Group::mark_member_for_addition(
         "Cannot get member; Group was not opened in write or modify_exclusive "
         "mode");
   }
-  return group_details_->mark_member_for_addition(
+  group_details_->mark_member_for_addition(
       group_member_uri, relative, name, storage_manager_);
+  return Status::Ok();
 }
 
-Status Group::mark_member_for_removal(const URI& uri) {
-  return mark_member_for_removal(uri.to_string());
-}
-
-Status Group::mark_member_for_removal(const std::string& uri) {
+Status Group::mark_member_for_removal(const std::string& name) {
   std::lock_guard<std::mutex> lck(mtx_);
   // Check if group is open
   if (!is_open_) {
@@ -599,7 +579,8 @@ Status Group::mark_member_for_removal(const std::string& uri) {
         "mode");
   }
 
-  return group_details_->mark_member_for_removal(uri);
+  group_details_->mark_member_for_removal(name);
+  return Status::Ok();
 }
 
 const std::vector<shared_ptr<GroupMember>>& Group::members_to_modify() const {
@@ -749,20 +730,20 @@ tuple<Status, optional<std::string>> Group::generate_name() const {
   return {Status::Ok(), ss.str()};
 }
 
-Status Group::load_metadata() {
+void Group::load_metadata() {
   if (remote_) {
     auto rest_client = storage_manager_->rest_client();
     if (rest_client == nullptr)
-      return Status_GroupError(
+      throw GroupStatusException(
           "Cannot load metadata; remote group with no REST client.");
-    RETURN_NOT_OK(rest_client->post_group_metadata_from_rest(group_uri_, this));
+    throw_if_not_ok(
+        rest_client->post_group_metadata_from_rest(group_uri_, this));
   } else {
     assert(group_dir_->loaded());
     storage_manager_->load_group_metadata(
         group_dir_, *encryption_key_, &metadata_);
   }
   metadata_loaded_ = true;
-  return Status::Ok();
 }
 
 }  // namespace sm
