@@ -46,6 +46,7 @@
 #include "tiledb/sm/stats/global_stats.h"
 #include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/subarray/subarray.h"
+#include "tiledb/type/apply_with_type.h"
 
 #include <numeric>
 
@@ -229,58 +230,24 @@ std::string DenseReader::name() {
 template <class OffType>
 Status DenseReader::dense_read() {
   auto type{array_schema_.domain().dimension_ptr(0)->type()};
-  switch (type) {
-    case Datatype::INT8:
-      return dense_read<int8_t, OffType>();
-    case Datatype::UINT8:
-      return dense_read<uint8_t, OffType>();
-    case Datatype::INT16:
-      return dense_read<int16_t, OffType>();
-    case Datatype::UINT16:
-      return dense_read<uint16_t, OffType>();
-    case Datatype::INT32:
-      return dense_read<int, OffType>();
-    case Datatype::UINT32:
-      return dense_read<unsigned, OffType>();
-    case Datatype::INT64:
-      return dense_read<int64_t, OffType>();
-    case Datatype::UINT64:
-      return dense_read<uint64_t, OffType>();
-    case Datatype::DATETIME_YEAR:
-    case Datatype::DATETIME_MONTH:
-    case Datatype::DATETIME_WEEK:
-    case Datatype::DATETIME_DAY:
-    case Datatype::DATETIME_HR:
-    case Datatype::DATETIME_MIN:
-    case Datatype::DATETIME_SEC:
-    case Datatype::DATETIME_MS:
-    case Datatype::DATETIME_US:
-    case Datatype::DATETIME_NS:
-    case Datatype::DATETIME_PS:
-    case Datatype::DATETIME_FS:
-    case Datatype::DATETIME_AS:
-    case Datatype::TIME_HR:
-    case Datatype::TIME_MIN:
-    case Datatype::TIME_SEC:
-    case Datatype::TIME_MS:
-    case Datatype::TIME_US:
-    case Datatype::TIME_NS:
-    case Datatype::TIME_PS:
-    case Datatype::TIME_FS:
-    case Datatype::TIME_AS:
-      return dense_read<int64_t, OffType>();
-    default:
-      return LOG_STATUS(Status_ReaderError(
-          "Cannot read dense array; Unsupported domain type"));
-  }
 
-  return Status::Ok();
+  auto g = [&](auto T) {
+    if constexpr (tiledb::type::TileDBIntegral<decltype(T)>) {
+      return dense_read<decltype(T), OffType>();
+    }
+    return Status_ReaderError(
+        "Cannot read dense array; Unsupported domain type");
+  };
+  return apply_with_type(g, type);
 }
 
 template <class DimType, class OffType>
 Status DenseReader::dense_read() {
   // Sanity checks.
-  assert(std::is_integral<DimType>::value);
+  if constexpr (!std::is_integral_v<DimType> || std::is_same_v<DimType, char>) {
+    throw StatusException(
+        Status_ReaderError("Cannot read dense array; Unsupported domain type"));
+  }
 
   // For easy reference.
   const auto dim_num = array_schema_.dim_num();
@@ -559,7 +526,6 @@ Status DenseReader::dense_read() {
           0,
           subarray_start_cell,
           subarray_end_cell,
-          0,
           nullptr,
           nullopt)};
       for (auto& aggregate : aggregates_[constants::count_of_rows]) {
@@ -1267,21 +1233,17 @@ AggregateBuffer DenseReader::make_aggregate_buffer(
     const uint64_t cell_size,
     const uint64_t min_cell,
     const uint64_t max_cell,
-    const uint64_t cell_num,
     ResultTile::TileTuple* tile_tuple,
     optional<void*> bitmap_data) {
   void* fixed_data = nullptr;
   optional<char*> var_data = nullopt;
   optional<uint8_t*> validity_data = nullopt;
-  uint64_t var_data_size = 0;
   if (tile_tuple != nullptr) {
     fixed_data =
         tile_tuple->fixed_tile().data_as<char>() + min_cell * cell_size;
     var_data = var_sized ?
                    std::make_optional(tile_tuple->var_tile().data_as<char>()) :
                    nullopt;
-    var_data_size =
-        var_sized && max_cell == cell_num ? tile_tuple->var_tile().size() : 0;
     validity_data =
         nullable ?
             std::make_optional(
@@ -1292,13 +1254,12 @@ AggregateBuffer DenseReader::make_aggregate_buffer(
   return AggregateBuffer(
       0,
       max_cell - min_cell,
-      cell_num - min_cell,
       fixed_data,
       var_data,
-      var_data_size,
       validity_data,
       false,
-      bitmap_data);
+      bitmap_data,
+      cell_size);
 }
 
 template <class DimType, class OffType>
@@ -1604,7 +1565,6 @@ Status DenseReader::copy_offset_tiles(
   // For easy reference
   const auto dim_num = array_schema_.dim_num();
   const auto cell_order = array_schema_.cell_order();
-  const auto cell_num_per_tile = array_schema_.domain().cell_num_per_tile();
   auto stride = array_schema_.domain().stride<DimType>(layout_);
   const auto& frag_domains = result_space_tile.frag_domains();
   auto dst_buf = static_cast<uint8_t*>(buffers_[name].buffer_);
@@ -1672,35 +1632,23 @@ Status DenseReader::copy_offset_tiles(
         const auto& t_var = tile_tuples[fd]->var_tile();
 
         // Setup variables for the copy.
-        auto src_buff =
-            static_cast<uint64_t*>(tile_tuples[fd]->fixed_tile().data()) +
-            start * stride + src_cell;
+        auto src_buff = tile_tuples[fd]->fixed_tile().data_as<offsets_t>() +
+                        start * stride + src_cell;
         auto div = elements_mode_ ? data_type_size : 1;
         auto dest = (OffType*)dest_ptr + start;
 
-        // Copy the data cell by cell, last copy was taken out to take
-        // advantage of vectorization.
+        // Copy the data cell by cell.
         uint64_t i = 0;
-        for (; i < end - start; ++i) {
+        for (; i <= end - start; ++i) {
           auto i_src = i * stride;
           dest[i] = (src_buff[i_src + 1] - src_buff[i_src]) / div;
           var_data_buff[i + start] = t_var.data_as<char>() + src_buff[i_src];
         }
 
-        // Copy the last value.
-        if (start + src_cell + (end - start) * stride >=
-            cell_num_per_tile - 1) {
-          dest[i] = (t_var.size() - src_buff[i * stride]) / div;
-        } else {
-          auto i_src = i * stride;
-          dest[i] = (src_buff[i_src + 1] - src_buff[i_src]) / div;
-        }
-        var_data_buff[i + start] = t_var.data_as<char>() + src_buff[i * stride];
-
         // Process validity values.
         if (nullable) {
           auto src_buff_validity =
-              static_cast<uint8_t*>(tile_tuples[fd]->validity_tile().data()) +
+              tile_tuples[fd]->validity_tile().data_as<uint8_t>() +
               start * stride + src_cell;
 
           for (i = 0; i < end - start + 1; ++i) {
@@ -1874,7 +1822,6 @@ Status DenseReader::aggregate_tiles(
   // For easy reference
   const auto dim_num = array_schema_.dim_num();
   const auto cell_order = array_schema_.cell_order();
-  const auto cell_num_per_tile = array_schema_.domain().cell_num_per_tile();
   auto stride = array_schema_.domain().stride<DimType>(layout_);
   const auto& frag_domains = result_space_tile.frag_domains();
   const auto attribute = array_schema_.attribute(name);
@@ -1940,7 +1887,6 @@ Status DenseReader::aggregate_tiles(
               cell_size,
               iter.pos_in_tile() + start,
               iter.pos_in_tile() + end + 1,
-              cell_num_per_tile,
               tile_tuples[fd],
               &aggregate_bitmap[cell_offset + start])};
           for (auto& aggregate : aggregates) {
@@ -1957,7 +1903,6 @@ Status DenseReader::aggregate_tiles(
                 cell_size,
                 start_cell,
                 start_cell + 1,
-                cell_num_per_tile,
                 tile_tuples[fd],
                 &aggregate_bitmap[cell_offset + start + i])};
             for (auto& aggregate : aggregates) {
