@@ -35,6 +35,7 @@
 #include "tiledb/sm/array_schema/domain.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
+#include "tiledb/type/apply_with_type.h"
 #include "tiledb/type/range/range.h"
 
 #include <cassert>
@@ -67,14 +68,16 @@ bool result_tile_cmp(const ResultTile* a, const ResultTile* b) {
 /* ****************************** */
 
 ResultTile::ResultTile(
-    unsigned frag_idx, uint64_t tile_idx, const ArraySchema& array_schema)
-    : domain_(&array_schema.domain())
+    unsigned frag_idx, uint64_t tile_idx, const FragmentMetadata& frag_md)
+    : domain_(&frag_md.array_schema()->domain())
     , frag_idx_(frag_idx)
-    , tile_idx_(tile_idx) {
+    , tile_idx_(tile_idx)
+    , cell_num_(frag_md.cell_num(tile_idx)) {
+  auto array_schema = frag_md.array_schema();
   coord_tiles_.resize(domain_->dim_num());
-  attr_tiles_.resize(array_schema.attribute_num());
-  for (uint64_t i = 0; i < array_schema.attribute_num(); i++) {
-    auto attribute = array_schema.attribute(i);
+  attr_tiles_.resize(array_schema->attribute_num());
+  for (uint64_t i = 0; i < array_schema->attribute_num(); i++) {
+    auto attribute = array_schema->attribute(i);
     attr_tiles_[i] = std::make_pair(attribute->name(), nullopt);
   }
   set_compute_results_func();
@@ -102,6 +105,7 @@ void ResultTile::swap(ResultTile& tile) {
   std::swap(domain_, tile.domain_);
   std::swap(frag_idx_, tile.frag_idx_);
   std::swap(tile_idx_, tile.tile_idx_);
+  std::swap(cell_num_, tile.cell_num_);
   std::swap(attr_tiles_, tile.attr_tiles_);
   std::swap(timestamps_tile_, tile.timestamps_tile_);
   std::swap(delete_timestamps_tile_, tile.delete_timestamps_tile_);
@@ -127,23 +131,7 @@ bool ResultTile::operator==(const ResultTile& rt) const {
 }
 
 uint64_t ResultTile::cell_num() const {
-  if (coord_tiles_[0].second.has_value()) {
-    return coord_tiles_[0].second->fixed_tile().cell_num();
-  }
-
-  if (coords_tile_.has_value()) {
-    return coords_tile_->fixed_tile().cell_num();
-  }
-
-  if (!attr_tiles_.empty()) {
-    for (const auto& attr : attr_tiles_) {
-      if (attr.second.has_value()) {
-        return attr.second.value().fixed_tile().cell_num();
-      }
-    }
-  }
-
-  return 0;
+  return cell_num_;
 }
 
 void ResultTile::erase_tile(const std::string& name) {
@@ -275,7 +263,7 @@ ResultTile::TileTuple* ResultTile::tile_tuple(const std::string& name) {
 const void* ResultTile::unzipped_coord(uint64_t pos, unsigned dim_idx) const {
   const auto& coord_tile = coord_tiles_[dim_idx].second->fixed_tile();
   const uint64_t offset = pos * coord_tile.cell_size();
-  void* const ret = static_cast<char*>(coord_tile.data()) + offset;
+  void* const ret = coord_tile.data_as<char>() + offset;
   return ret;
 }
 
@@ -284,8 +272,7 @@ const void* ResultTile::zipped_coord(uint64_t pos, unsigned dim_idx) const {
   auto coord_size =
       coords_size / coords_tile_->fixed_tile().zipped_coords_dim_num();
   const uint64_t offset = pos * coords_size + dim_idx * coord_size;
-  void* const ret =
-      static_cast<char*>(coords_tile_->fixed_tile().data()) + offset;
+  void* const ret = coords_tile_->fixed_tile().data_as<char>() + offset;
   return ret;
 }
 
@@ -293,26 +280,20 @@ std::string_view ResultTile::coord_string(
     uint64_t pos, unsigned dim_idx) const {
   const auto& coord_tile_off = coord_tiles_[dim_idx].second->fixed_tile();
   const auto& coord_tile_val = coord_tiles_[dim_idx].second->var_tile();
-  auto cell_num = coord_tile_off.cell_num();
-  auto val_size = coord_tile_val.size();
 
-  uint64_t offset = 0;
+  offsets_t offset = 0;
   Status st =
       coord_tile_off.read(&offset, pos * sizeof(uint64_t), sizeof(uint64_t));
   assert(st.ok());
 
-  uint64_t next_offset = 0;
-  if (pos == cell_num - 1) {
-    next_offset = val_size;
-  } else {
-    st = coord_tile_off.read(
-        &next_offset, (pos + 1) * sizeof(uint64_t), sizeof(uint64_t));
-    assert(st.ok());
-  }
+  offsets_t next_offset = 0;
+  st = coord_tile_off.read(
+      &next_offset, (pos + 1) * sizeof(uint64_t), sizeof(uint64_t));
+  assert(st.ok());
 
   auto size = next_offset - offset;
 
-  auto* buffer = static_cast<char*>(coord_tile_val.data()) + offset;
+  auto* buffer = coord_tile_val.data_as<char>() + offset;
   return std::string_view(buffer, size);
 }
 
@@ -365,13 +346,11 @@ template <>
 std::string_view ResultTile::attribute_value<std::string_view>(
     const std::string& label_name, const uint64_t pos) {
   auto tuple = tile_tuple(label_name);
-  auto offsets_data = tuple->fixed_tile().data_as<uint64_t>();
+  auto offsets_data = tuple->fixed_tile().data_as<offsets_t>();
   auto& var_tile = tuple->var_tile();
   auto offset = offsets_data[pos];
 
-  auto size = static_cast<size_t>(pos) == cell_num() - 1 ?
-                  var_tile.size() - offset :
-                  offsets_data[pos + 1] - offset;
+  auto size = offsets_data[pos + 1] - offset;
   return std::string_view(&var_tile.data_as<char>()[offset], size);
 }
 
@@ -528,7 +507,7 @@ void ResultTile::compute_results_dense(
   if (!stores_zipped_coords) {
     const auto& coord_tile = result_tile->coord_tile(dim_idx).fixed_tile();
 
-    auto coords = (const T*)coord_tile.data();
+    auto coords = coord_tile.data_as<const T>();
     T c;
 
     if (dim_idx == dim_num - 1) {
@@ -547,8 +526,7 @@ void ResultTile::compute_results_dense(
               for (unsigned d = 0; d < dim_num; ++d) {
                 const auto& coord_tile =
                     result_tile->coord_tile(dim_idx).fixed_tile();
-                const T* const coords =
-                    static_cast<const T*>(coord_tile.data());
+                const T* const coords = coord_tile.data_as<const T>();
                 auto c_d = coords[pos];
                 auto dom = (const T*)meta->non_empty_domain()[d].data();
                 if (c_d < dom[0] || c_d > dom[1]) {
@@ -574,7 +552,7 @@ void ResultTile::compute_results_dense(
   // Handle zipped coordinates tile
   assert(stores_zipped_coords);
   const auto& coords_tile = result_tile->zipped_coords_tile();
-  auto coords = (const T*)coords_tile.data();
+  auto coords = coords_tile.data_as<const T>();
   T c;
 
   if (dim_idx == dim_num - 1) {
@@ -644,12 +622,11 @@ void ResultTile::compute_results_sparse<char>(
 
   // Get offset buffer
   const auto& coord_tile_off = coord_tile.fixed_tile();
-  auto buff_off = static_cast<const uint64_t*>(coord_tile_off.data());
+  auto buff_off = coord_tile_off.data_as<offsets_t>();
 
   // Get string buffer
   const auto& coord_tile_str = coord_tile.var_tile();
-  auto buff_str = static_cast<const char*>(coord_tile_str.data());
-  auto buff_str_size = coord_tile_str.size();
+  auto buff_str = coord_tile_str.data_as<char>();
 
   // For row-major cell orders, the first dimension is sorted.
   // For col-major cell orders, the last dimension is sorted.
@@ -695,12 +672,10 @@ void ResultTile::compute_results_sparse<char>(
       // The coordinate values are determined by their offset and size
       // within `buff_str`. Calculate the offset and size for the
       // first and last coordinates in this partition.
-      const uint64_t first_c_offset = buff_off[first_c_pos];
-      const uint64_t last_c_offset = buff_off[last_c_pos];
-      const uint64_t first_c_size = buff_off[first_c_pos + 1] - first_c_offset;
-      const uint64_t last_c_size = (last_c_pos == coords_num - 1) ?
-                                       buff_str_size - last_c_offset :
-                                       buff_off[last_c_pos + 1] - last_c_offset;
+      const offsets_t first_c_offset = buff_off[first_c_pos];
+      const offsets_t last_c_offset = buff_off[last_c_pos];
+      const offsets_t first_c_size = buff_off[first_c_pos + 1] - first_c_offset;
+      const offsets_t last_c_size = buff_off[last_c_pos + 1] - last_c_offset;
 
       // Fetch the coordinate values for the first and last coordinates
       // in this partition.
@@ -717,11 +692,11 @@ void ResultTile::compute_results_sparse<char>(
         memset(&r_bitmap[first_c_pos], intersects, c_partition_size);
       } else {
         // Compute results
-        uint64_t c_offset = 0, c_size = 0;
+        offsets_t c_offset = 0;
+        uint64_t c_size = 0;
         for (uint64_t pos = first_c_pos; pos <= last_c_pos; ++pos) {
           c_offset = buff_off[pos];
-          c_size = (pos < coords_num - 1) ? buff_off[pos + 1] - c_offset :
-                                            buff_str_size - c_offset;
+          c_size = buff_off[pos + 1] - c_offset;
           r_bitmap[pos] = str_coord_intersects(
               c_offset, c_size, buff_str, range_start, range_end);
         }
@@ -774,8 +749,7 @@ void ResultTile::compute_results_sparse<char>(
         continue;
 
       c_offset = buff_off[pos];
-      c_size = (pos < coords_num - 1) ? buff_off[pos + 1] - c_offset :
-                                        buff_str_size - c_offset;
+      c_size = buff_off[pos + 1] - c_offset;
       r_bitmap[pos] = str_coord_intersects(
           c_offset, c_size, buff_str, range_start, range_end);
     }
@@ -788,10 +762,7 @@ void ResultTile::compute_results_sparse(
     unsigned dim_idx,
     const Range& range,
     std::vector<uint8_t>* result_bitmap,
-    const Layout& cell_order) {
-  // We do not use `cell_order` for this template type.
-  (void)cell_order;
-
+    const Layout&) {
   // For easy reference.
   auto coords_num = result_tile->cell_num();
   auto r = (const T*)range.data();
@@ -807,7 +778,7 @@ void ResultTile::compute_results_sparse(
   // Handle separate coordinate tiles
   if (!stores_zipped_coords) {
     const auto& coord_tile = result_tile->coord_tile(dim_idx).fixed_tile();
-    const T* const coords = static_cast<const T*>(coord_tile.data());
+    const T* const coords = coord_tile.data_as<const T>();
     for (uint64_t pos = 0; pos < coords_num; ++pos) {
       c = coords[pos];
       r_bitmap[pos] &= (uint8_t)(c >= r0 && c <= r1);
@@ -819,7 +790,7 @@ void ResultTile::compute_results_sparse(
   // Handle zipped coordinates tile
   assert(stores_zipped_coords);
   const auto& coords_tile = result_tile->zipped_coords_tile();
-  const T* const coords = static_cast<const T*>(coords_tile.data());
+  const T* const coords = coords_tile.data_as<const T>();
   for (uint64_t pos = 0; pos < coords_num; ++pos) {
     c = coords[pos * dim_num + dim_idx];
     r_bitmap[pos] &= (uint8_t)(c >= r0 && c <= r1);
@@ -831,9 +802,7 @@ void ResultTile::compute_results_count_sparse_string_range(
     const std::vector<std::pair<std::string_view, std::string_view>>
         cached_ranges,
     const char* buff_str,
-    const uint64_t* buff_off,
-    const uint64_t cell_num,
-    const uint64_t buff_str_size,
+    const offsets_t* buff_off,
     const uint64_t start,
     const uint64_t end,
     std::vector<BitmapType>& result_count) {
@@ -844,9 +813,8 @@ void ResultTile::compute_results_count_sparse_string_range(
     if (result_count[pos] == 0)
       continue;
 
-    uint64_t c_offset = buff_off[pos];
-    uint64_t c_size = (pos < cell_num - 1) ? buff_off[pos + 1] - c_offset :
-                                             buff_str_size - c_offset;
+    offsets_t c_offset = buff_off[pos];
+    uint64_t c_size = buff_off[pos + 1] - c_offset;
 
     const std::string_view str(buff_str + c_offset, c_size);
 
@@ -907,7 +875,6 @@ void ResultTile::compute_results_count_sparse_string(
     const Layout& cell_order,
     const uint64_t min_cell,
     const uint64_t max_cell) {
-  auto cell_num = result_tile->cell_num();
   auto coords_num = max_cell - min_cell;
   auto dim_num = result_tile->domain()->dim_num();
 
@@ -921,12 +888,11 @@ void ResultTile::compute_results_count_sparse_string(
 
   // Get offset buffer
   const auto& coord_tile_off = coord_tile.fixed_tile();
-  auto buff_off = static_cast<const uint64_t*>(coord_tile_off.data());
+  auto buff_off = coord_tile_off.data_as<offsets_t>();
 
   // Get string buffer
   const auto& coord_tile_str = coord_tile.var_tile();
-  auto buff_str = static_cast<const char*>(coord_tile_str.data());
-  auto buff_str_size = coord_tile_str.size();
+  auto buff_str = coord_tile_str.data_as<char>();
 
   // Cache start_str/end_str for all ranges.
   std::vector<std::pair<std::string_view, std::string_view>> cached_ranges;
@@ -980,12 +946,10 @@ void ResultTile::compute_results_count_sparse_string(
       // The coordinate values are determined by their offset and size
       // within `buff_str`. Calculate the offset and size for the
       // first and last coordinates in this partition.
-      const uint64_t first_c_offset = buff_off[first_c_pos];
-      const uint64_t last_c_offset = buff_off[last_c_pos];
-      const uint64_t first_c_size = buff_off[first_c_pos + 1] - first_c_offset;
-      const uint64_t last_c_size = (last_c_pos == cell_num - 1) ?
-                                       buff_str_size - last_c_offset :
-                                       buff_off[last_c_pos + 1] - last_c_offset;
+      const offsets_t first_c_offset = buff_off[first_c_pos];
+      const offsets_t last_c_offset = buff_off[last_c_pos];
+      const offsets_t first_c_size = buff_off[first_c_pos + 1] - first_c_offset;
+      const offsets_t last_c_size = buff_off[last_c_pos + 1] - last_c_offset;
 
       // Fetch the coordinate values for the first and last coordinates
       // in this partition.
@@ -1016,8 +980,6 @@ void ResultTile::compute_results_count_sparse_string(
             cached_ranges,
             buff_str,
             buff_off,
-            cell_num,
-            buff_str_size,
             first_c_pos,
             last_c_pos,
             result_count);
@@ -1072,8 +1034,6 @@ void ResultTile::compute_results_count_sparse_string(
           cached_ranges,
           buff_str,
           buff_off,
-          cell_num,
-          buff_str_size,
           i,
           i + partition_size - 1,
           result_count);
@@ -1088,12 +1048,9 @@ void ResultTile::compute_results_count_sparse(
     const NDRange& ranges,
     const std::vector<uint64_t>& range_indexes,
     std::vector<BitmapType>& result_count,
-    const Layout& cell_order,
+    const Layout&,
     const uint64_t min_cell,
     const uint64_t max_cell) {
-  // We don't use cell_order for this template type.
-  (void)cell_order;
-
   // For easy reference.
   auto stores_zipped_coords = result_tile->stores_zipped_coords();
   auto dim_num = result_tile->domain()->dim_num();
@@ -1101,7 +1058,7 @@ void ResultTile::compute_results_count_sparse(
   // Handle separate coordinate tiles.
   if (!stores_zipped_coords) {
     const auto& coord_tile = result_tile->coord_tile(dim_idx).fixed_tile();
-    const T* const coords = static_cast<const T*>(coord_tile.data());
+    const T* const coords = coord_tile.data_as<const T>();
     {
       // Iterate over all cells.
       for (uint64_t pos = min_cell; pos < max_cell; ++pos) {
@@ -1157,7 +1114,7 @@ void ResultTile::compute_results_count_sparse(
   // Handle zipped coordinates tile.
   assert(stores_zipped_coords);
   const auto& coords_tile = result_tile->zipped_coords_tile();
-  const T* const coords = static_cast<const T*>(coords_tile.data());
+  const T* const coords = coords_tile.data_as<const T>();
   {
     for (uint64_t pos = min_cell; pos < max_cell; ++pos) {
       if (result_count[pos]) {
@@ -1259,133 +1216,29 @@ void ResultTile::set_compute_results_func() {
   compute_results_sparse_func_.resize(dim_num);
   compute_results_count_sparse_uint8_t_func_.resize(dim_num);
   compute_results_count_sparse_uint64_t_func_.resize(dim_num);
+
   for (unsigned d = 0; d < dim_num; ++d) {
     auto dim{domain_->dimension_ptr(d)};
-    switch (dim->type()) {
-      case Datatype::INT32:
-        compute_results_dense_func_[d] = compute_results_dense<int32_t>;
-        compute_results_sparse_func_[d] = compute_results_sparse<int32_t>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, int32_t>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, int32_t>;
-        break;
-      case Datatype::INT64:
-        compute_results_dense_func_[d] = compute_results_dense<int64_t>;
-        compute_results_sparse_func_[d] = compute_results_sparse<int64_t>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, int64_t>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, int64_t>;
-        break;
-      case Datatype::INT8:
-        compute_results_dense_func_[d] = compute_results_dense<int8_t>;
-        compute_results_sparse_func_[d] = compute_results_sparse<int8_t>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, int8_t>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, int8_t>;
-        break;
-      case Datatype::UINT8:
-        compute_results_dense_func_[d] = compute_results_dense<uint8_t>;
-        compute_results_sparse_func_[d] = compute_results_sparse<uint8_t>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, uint8_t>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, uint8_t>;
-        break;
-      case Datatype::INT16:
-        compute_results_dense_func_[d] = compute_results_dense<int16_t>;
-        compute_results_sparse_func_[d] = compute_results_sparse<int16_t>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, int16_t>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, int16_t>;
-        break;
-      case Datatype::UINT16:
-        compute_results_dense_func_[d] = compute_results_dense<uint16_t>;
-        compute_results_sparse_func_[d] = compute_results_sparse<uint16_t>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, uint16_t>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, uint16_t>;
-        break;
-      case Datatype::UINT32:
-        compute_results_dense_func_[d] = compute_results_dense<uint32_t>;
-        compute_results_sparse_func_[d] = compute_results_sparse<uint32_t>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, uint32_t>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, uint32_t>;
-        break;
-      case Datatype::UINT64:
-        compute_results_dense_func_[d] = compute_results_dense<uint64_t>;
-        compute_results_sparse_func_[d] = compute_results_sparse<uint64_t>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, uint64_t>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, uint64_t>;
-        break;
-      case Datatype::FLOAT32:
-        compute_results_dense_func_[d] = compute_results_dense<float>;
-        compute_results_sparse_func_[d] = compute_results_sparse<float>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, float>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, float>;
-        break;
-      case Datatype::FLOAT64:
-        compute_results_dense_func_[d] = compute_results_dense<double>;
-        compute_results_sparse_func_[d] = compute_results_sparse<double>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, double>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, double>;
-        break;
-      case Datatype::DATETIME_YEAR:
-      case Datatype::DATETIME_MONTH:
-      case Datatype::DATETIME_WEEK:
-      case Datatype::DATETIME_DAY:
-      case Datatype::DATETIME_HR:
-      case Datatype::DATETIME_MIN:
-      case Datatype::DATETIME_SEC:
-      case Datatype::DATETIME_MS:
-      case Datatype::DATETIME_US:
-      case Datatype::DATETIME_NS:
-      case Datatype::DATETIME_PS:
-      case Datatype::DATETIME_FS:
-      case Datatype::DATETIME_AS:
-      case Datatype::TIME_HR:
-      case Datatype::TIME_MIN:
-      case Datatype::TIME_SEC:
-      case Datatype::TIME_MS:
-      case Datatype::TIME_US:
-      case Datatype::TIME_NS:
-      case Datatype::TIME_PS:
-      case Datatype::TIME_FS:
-      case Datatype::TIME_AS:
-        compute_results_dense_func_[d] = compute_results_dense<int64_t>;
-        compute_results_sparse_func_[d] = compute_results_sparse<int64_t>;
-        compute_results_count_sparse_uint8_t_func_[d] =
-            compute_results_count_sparse<uint8_t, int64_t>;
-        compute_results_count_sparse_uint64_t_func_[d] =
-            compute_results_count_sparse<uint64_t, int64_t>;
-        break;
-      case Datatype::STRING_ASCII:
+
+    auto g = [&](auto T) {
+      if constexpr (std::is_same_v<decltype(T), char>) {
         compute_results_dense_func_[d] = nullptr;
-        compute_results_sparse_func_[d] = compute_results_sparse<char>;
+        compute_results_sparse_func_[d] = compute_results_sparse<decltype(T)>;
         compute_results_count_sparse_uint8_t_func_[d] =
             compute_results_count_sparse_string<uint8_t>;
         compute_results_count_sparse_uint64_t_func_[d] =
             compute_results_count_sparse_string<uint64_t>;
-        break;
-      default:
-        compute_results_dense_func_[d] = nullptr;
-        compute_results_sparse_func_[d] = nullptr;
-        compute_results_count_sparse_uint8_t_func_[d] = nullptr;
-        compute_results_count_sparse_uint64_t_func_[d] = nullptr;
-        break;
-    }
+        return;
+      } else if constexpr (tiledb::type::TileDBFundamental<decltype(T)>) {
+        compute_results_dense_func_[d] = compute_results_dense<decltype(T)>;
+        compute_results_sparse_func_[d] = compute_results_sparse<decltype(T)>;
+        compute_results_count_sparse_uint8_t_func_[d] =
+            compute_results_count_sparse<uint8_t, decltype(T)>;
+        compute_results_count_sparse_uint64_t_func_[d] =
+            compute_results_count_sparse<uint64_t, decltype(T)>;
+      }
+    };
+    apply_with_type(g, dim->type());
   }
 }
 
