@@ -606,17 +606,14 @@ void SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
     uint8_t* val_buffer,
     const void** var_data) {
   // Get source buffers.
-  const auto cell_num =
-      fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
   const auto tile_tuple = rt->tile_tuple(name);
 
   // If the tile_tuple is null, this is a field added in schema
   // evolution. Use the fill value.
-  const uint64_t* src_buff = nullptr;
+  const offsets_t* src_buff = nullptr;
   const uint8_t* src_var_buff = nullptr;
   bool use_fill_value = false;
   OffType fill_value_size = 0;
-  uint64_t t_var_size = 0;
   if (tile_tuple == nullptr) {
     use_fill_value = true;
     fill_value_size = static_cast<OffType>(
@@ -625,17 +622,13 @@ void SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
   } else {
     const auto& t = tile_tuple->fixed_tile();
     const auto& t_var = tile_tuple->var_tile();
-    t_var_size = t_var.size();
-    src_buff = t.template data_as<uint64_t>();
+    src_buff = t.template data_as<offsets_t>();
     src_var_buff = t_var.template data_as<uint8_t>();
   }
 
-  // Process all cells. Last cell might be taken out for vectorization.
-  uint64_t end = (src_max_pos == cell_num && !use_fill_value) ?
-                     src_max_pos - 1 :
-                     src_max_pos;
+  // Process all cells.
   if (!use_fill_value) {
-    for (uint64_t c = src_min_pos; c < end; c++) {
+    for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
       for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
         *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
         buffer++;
@@ -643,19 +636,8 @@ void SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
         var_data++;
       }
     }
-
-    // Do last cell.
-    if (src_max_pos == cell_num) {
-      for (uint64_t i = 0; i < rt->bitmap()[src_max_pos - 1]; i++) {
-        *buffer =
-            (OffType)(t_var_size - src_buff[src_max_pos - 1]) / offset_div;
-        buffer++;
-        *var_data = src_var_buff + src_buff[src_max_pos - 1];
-        var_data++;
-      }
-    }
   } else {
-    for (uint64_t c = src_min_pos; c < end; c++) {
+    for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
       for (uint64_t i = 0; i < rt->bitmap()[c]; i++) {
         *buffer = fill_value_size / offset_div;
         buffer++;
@@ -702,17 +684,15 @@ void SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
     uint8_t* val_buffer,
     const void** var_data) {
   // Get source buffers.
-  const auto cell_num =
-      fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
   const auto tile_tuple = rt->tile_tuple(name);
 
   // If the tile_tuple is null, this is a field added in schema
   // evolution. Use the fill value.
-  const uint64_t* src_buff = nullptr;
+  const offsets_t* src_buff = nullptr;
   const uint8_t* src_var_buff = nullptr;
   bool use_fill_value = false;
   OffType fill_value_size = 0;
-  uint64_t t_var_size = 0;
+
   if (tile_tuple == nullptr) {
     use_fill_value = true;
     fill_value_size = static_cast<OffType>(
@@ -721,18 +701,14 @@ void SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
   } else {
     const auto& t = tile_tuple->fixed_tile();
     const auto& t_var = tile_tuple->var_tile();
-    t_var_size = t_var.size();
-    src_buff = t.template data_as<uint64_t>();
+    src_buff = t.template data_as<offsets_t>();
     src_var_buff = t_var.template data_as<uint8_t>();
   }
 
   if (!rt->copy_full_tile() || use_fill_value) {
-    // Process all cells. Last cell might be taken out for vectorization.
-    uint64_t end = (src_max_pos == cell_num && !use_fill_value) ?
-                       src_max_pos - 1 :
-                       src_max_pos;
+    // Process all cells.
     if (!use_fill_value) {
-      for (uint64_t c = src_min_pos; c < end; c++) {
+      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
         if (rt->bitmap()[c]) {
           *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
           buffer++;
@@ -740,15 +716,8 @@ void SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
           var_data++;
         }
       }
-
-      // Do last cell.
-      if (src_max_pos == cell_num && rt->bitmap()[src_max_pos - 1]) {
-        *buffer =
-            (OffType)(t_var_size - src_buff[src_max_pos - 1]) / offset_div;
-        *var_data = src_var_buff + src_buff[src_max_pos - 1];
-      }
     } else {
-      for (uint64_t c = src_min_pos; c < end; c++) {
+      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
         if (!rt->has_bmp() || rt->bitmap()[c]) {
           *buffer = fill_value_size / offset_div;
           buffer++;
@@ -780,21 +749,14 @@ void SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
       }
     }
   } else {
-    // Copy full tile. Last cell might be taken out for vectorization.
-    uint64_t end = (src_max_pos == cell_num) ? src_max_pos - 1 : src_max_pos;
-    for (uint64_t c = src_min_pos; c < end; c++) {
+    // Copy full tile.
+    for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
       *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
       buffer++;
       *var_data = src_var_buff + src_buff[c];
       var_data++;
     }
 
-    // Copy last cell.
-    if (src_max_pos == cell_num) {
-      *buffer = (OffType)(t_var_size - src_buff[src_max_pos - 1]) / offset_div;
-      *var_data = src_var_buff + src_buff[src_max_pos - 1];
-    }
-
     // Copy nullable values.
     if (nullable) {
       const auto& t_val = tile_tuple->validity_tile();
@@ -1912,15 +1874,14 @@ SparseUnorderedWithDupsReader<BitmapType>::make_aggregate_buffer(
     const std::string name,
     const bool var_sized,
     const bool nullable,
+    const uint64_t cell_size,
     const bool count_bitmap,
     const uint64_t min_cell,
     const uint64_t max_cell,
-    const uint64_t cell_num,
     UnorderedWithDupsResultTile<BitmapType>& rt) {
   return AggregateBuffer(
       min_cell,
       max_cell,
-      cell_num,
       name == constants::count_of_rows ?
           nullptr :
           rt.tile_tuple(name)->fixed_tile().data(),
@@ -1928,16 +1889,14 @@ SparseUnorderedWithDupsReader<BitmapType>::make_aggregate_buffer(
           std::make_optional(
               rt.tile_tuple(name)->var_tile().template data_as<char>()) :
           nullopt,
-      var_sized && max_cell == cell_num ?
-          rt.tile_tuple(name)->var_tile().size() :
-          0,
       nullable ? std::make_optional(rt.tile_tuple(name)
                                         ->validity_tile()
                                         .template data_as<uint8_t>()) :
                  nullopt,
       count_bitmap,
       rt.bitmap().data() != nullptr ? std::make_optional(rt.bitmap().data()) :
-                                      nullopt);
+                                      nullopt,
+      cell_size);
 }
 
 template <class BitmapType>
@@ -1950,10 +1909,12 @@ void SparseUnorderedWithDupsReader<BitmapType>::process_aggregates(
 
   bool var_sized = false;
   bool nullable = false;
+  unsigned cell_val_num = 0;
 
   if (name != constants::count_of_rows) {
     var_sized = array_schema_.var_size(name);
     nullable = array_schema_.is_nullable(name);
+    cell_val_num = array_schema_.cell_val_num(name);
   }
 
   const bool count_bitmap = std::is_same<BitmapType, uint64_t>::value;
@@ -1977,21 +1938,12 @@ void SparseUnorderedWithDupsReader<BitmapType>::process_aggregates(
 
         uint64_t cell_num =
             fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
-        uint64_t min_pos_tile = 0;
-        uint64_t max_pos_tile = cell_num;
-        // Adjust max cell if this is the last tile.
-        if (i == result_tiles.size() - 1) {
-          uint64_t to_copy = cell_offsets[i + 1] - cell_offsets[i];
-          max_pos_tile =
-              rt->pos_with_given_result_sum(min_pos_tile, to_copy) + 1;
-        }
-
         auto&& [skip_aggregate, src_min_pos, src_max_pos, dest_cell_offset] =
             compute_parallelization_parameters(
                 range_thread_idx,
                 num_range_threads,
-                min_pos_tile,
-                max_pos_tile,
+                0,
+                cell_num,
                 cell_offsets[i],
                 nullptr);
         if (skip_aggregate) {
@@ -2003,10 +1955,10 @@ void SparseUnorderedWithDupsReader<BitmapType>::process_aggregates(
             name,
             var_sized,
             nullable,
+            cell_val_num,
             count_bitmap,
             src_min_pos,
             src_max_pos,
-            cell_num,
             *rt)};
         for (auto& aggregate : aggregates) {
           aggregate->aggregate_data(aggregate_buffer);
