@@ -496,20 +496,20 @@ class SparseUnorderedWithDupsReader : public SparseIndexReaderBase,
    * @param name Field to aggregate.
    * @param var_sized Is the field var sized?
    * @param nullable Is the field nullable?
+   * @param cell_size Cell size for the field.
    * @param count_bitmap Is the bitmap a count bitmap?
    * @param min_cell Min cell to aggregate.
    * @param min_cell Max cell to aggregate.
-   * @param cell_num Number of cells for the tile
    * @param rt Result tile.
    */
   AggregateBuffer make_aggregate_buffer(
       const std::string name,
       const bool var_sized,
       const bool nullable,
+      const uint64_t cell_size,
       const bool count_bitmap,
       const uint64_t min_cell,
       const uint64_t max_cell,
-      const uint64_t cell_num,
       UnorderedWithDupsResultTile<BitmapType>& rt);
 
   /**
