@@ -58,6 +58,13 @@ namespace tiledb {
 namespace sm {
 namespace serialization {
 
+class ArraySerializationException : public StatusException {
+ public:
+  explicit ArraySerializationException(const std::string& message)
+      : StatusException("[TileDB::Serialization][Array]", message) {
+  }
+};
+
 #ifdef TILEDB_SERIALIZATION
 
 Status metadata_to_capnp(
@@ -108,9 +115,9 @@ Status metadata_from_capnp(
     }
 
     if (entry_reader.getDel()) {
-      RETURN_NOT_OK(metadata->del(key.c_str()));
+      metadata->del(key.c_str());
     } else {
-      RETURN_NOT_OK(metadata->put(key.c_str(), type, value_num, value));
+      metadata->put(key.c_str(), type, value_num, value);
     }
   }
 
@@ -126,7 +133,7 @@ Status array_to_capnp(
   // want to serialized a query object TileDB >= 2.5 no longer needs to send the
   // array URI
   if (!array->array_uri_serialized().to_string().empty()) {
-    array_builder->setUri(array->array_uri_serialized());
+    array_builder->setUri(array->array_uri_serialized().to_string());
   }
   array_builder->setStartTimestamp(array->timestamp_start());
   array_builder->setEndTimestamp(array->timestamp_end());
@@ -697,7 +704,7 @@ Status array_serialize(Array*, SerializationType, Buffer*, const bool) {
 Status array_deserialize(
     Array*, SerializationType, const Buffer&, StorageManager*) {
   return LOG_STATUS(Status_SerializationError(
-      "Cannot serialize; serialization not enabled."));
+      "Cannot deserialize; serialization not enabled."));
 }
 
 Status array_open_serialize(const Array&, SerializationType, Buffer*) {
@@ -717,7 +724,7 @@ Status metadata_serialize(Metadata*, SerializationType, Buffer*) {
 
 Status metadata_deserialize(Metadata*, SerializationType, const Buffer&) {
   return LOG_STATUS(Status_SerializationError(
-      "Cannot serialize; serialization not enabled."));
+      "Cannot deserialize; serialization not enabled."));
 }
 
 #endif  // TILEDB_SERIALIZATION
